/*
 * This header provides constants for netxbig LED bindings.
 *
 * This file is licensed under the terms of the GNU General Public
 * License version 2.  This program is licensed "as is" without any
 * warranty of any kind, whether express or implied.
 */

#ifndef _DT_BINDINGS_LEDS_NETXBIG_H
#define _DT_BINDINGS_LEDS_NETXBIG_H

#define NETXBIG_LED_OFF		0
#define NETXBIG_LED_ON		1
#define NETXBIG_LED_SATA	2
#define NETXBIG_LED_TIMER1	3
#define NETXBIG_LED_TIMER2	4

#endif /* _DT_BINDINGS_LEDS_NETXBIG_H */
