/* SPDX-License-Identifier: (GPL-2.0-only OR BSD-2-Clause) */

#ifndef _DT_BINDINGS_POWER_SUN20I_D1_PPU_H_
#define _DT_BINDINGS_POWER_SUN20I_D1_PPU_H_

#define PD_CPU			0
#define PD_VE			1
#define PD_DSP			2

#endif /* _DT_BINDINGS_POWER_SUN20I_D1_PPU_H_ */
