/* SPDX-License-Identifier: GPL-2.0-only OR BSD-2-Clause */
/*
 * Copyright (C) 2024 Inochi Amaoto <inochiama@outlook.com>
 *
 * This file is generated from vendor pinout definition.
 */

#ifndef _DT_BINDINGS_PINCTRL_CV1800B_H
#define _DT_BINDINGS_PINCTRL_CV1800B_H

#include <dt-bindings/pinctrl/pinctrl-cv18xx.h>

#define PIN_AUD_AOUTR			1
#define PIN_SD0_CLK			3
#define PIN_SD0_CMD			4
#define PIN_SD0_D0			5
#define PIN_SD0_D1			7
#define PIN_SD0_D2			8
#define PIN_SD0_D3			9
#define PIN_SD0_CD			11
#define PIN_SD0_PWR_EN			12
#define PIN_SPK_EN			14
#define PIN_UART0_TX			15
#define PIN_UART0_RX			16
#define PIN_SPINOR_HOLD_X		17
#define PIN_SPINOR_SCK			18
#define PIN_SPINOR_MOSI			19
#define PIN_SPINOR_WP_X			20
#define PIN_SPINOR_MISO			21
#define PIN_SPINOR_CS_X			22
#define PIN_IIC0_SCL			23
#define PIN_IIC0_SDA			24
#define PIN_AUX0			25
#define PIN_PWR_VBAT_DET		30
#define PIN_PWR_SEQ2			31
#define PIN_XTAL_XIN			33
#define PIN_SD1_GPIO0			35
#define PIN_SD1_GPIO1			36
#define PIN_SD1_D3			38
#define PIN_SD1_D2			39
#define PIN_SD1_D1			40
#define PIN_SD1_D0			41
#define PIN_SD1_CMD			42
#define PIN_SD1_CLK			43
#define PIN_ADC1			44
#define PIN_USB_VBUS_DET		45
#define PIN_ETH_TXP			47
#define PIN_ETH_TXM			48
#define PIN_ETH_RXP			49
#define PIN_ETH_RXM			50
#define PIN_MIPIRX4N			56
#define PIN_MIPIRX4P			57
#define PIN_MIPIRX3N			58
#define PIN_MIPIRX3P			59
#define PIN_MIPIRX2N			60
#define PIN_MIPIRX2P			61
#define PIN_MIPIRX1N			62
#define PIN_MIPIRX1P			63
#define PIN_MIPIRX0N			64
#define PIN_MIPIRX0P			65
#define PIN_AUD_AINL_MIC		67

#endif /* _DT_BINDINGS_PINCTRL_CV1800B_H */
